using System.Collections.Generic;
using System.Linq;
using UnityEngine;
using System;

//////--------------------------------------------------------------------------------//
//////            Автор плагина Vladimir.Kzi / Plugin autor Vladimir.Kzi              //
//////--------------------------------------------------------------------------------//
//////  Поддержка данного плагина: www.oxide-russia.ru and www.vk.com/deviantplugins  //
//////--------------------------------------------------------------------------------//
//////  Support for this plugin: www.oxide-russia.ru и www.vk.com/deviantplugins      //
//////--------------------------------------------------------------------------------//
//////                     Pomoc dla tego pluginu: oxidepolska.pl                     //
//////--------------------------------------------------------------------------------//
namespace Oxide.Plugins
{
    [Info("EventFarming", "Vladimir.Kzi", "0.0.9")]
    [Description("Farming resources mini-event(AutoEvent).")]
    class EventFarming : HurtworldPlugin
    {
		
		//////////////////////////////////////////////////////////////////////////////////////////
        // Классы
        //////////////////////////////////////////////////////////////////////////////////////////
		
		private ConfigData configData;
		public bool farmStarted;
		public DateTime startTime;
		public Dictionary<ulong, int> playersFarming = new Dictionary<ulong, int>();
		private Dictionary<string, DateTime> playersWin = new Dictionary<string, DateTime>();
		private FarmEvent currentFarmEvent = new FarmEvent();
		
		class ConfigData
        {
            public bool autoStart { get; set; }
			public float intervalAutoStart { get; set; }
			public float chatProgressInterval { get; set; }
            public int minPlayers { get; set; }
			public bool giveGift { get; set; }
            public bool giveRandomAllGift { get; set; }
			public bool showWinnerInChat { get; set; }
			public bool writeWinLog { get; set; }
			public bool showWinConsole { get; set; }
            public List<GiftItems> listGiftItems { get; set; }
            public List<FarmEvents> listFarmEvents { get; set; }
        }
		
		class GiftItems
        {
            public string itemName;
            public int itemID;
            public int itemCount;
        }
		
		class FarmEvents
        {
            public string Name;
            public string Item;
            public float Time;
            public int Farm;
            public int Places;
        }
		
		class FarmEvent
        {
            public string Name;
            public string Item;
            public Timer Time;
			public Timer Progress;
            public int Farm;
            public int Places;
        }
		
		//////////////////////////////////////////////////////////////////////////////////////////
        // Загрузка плагина
        //////////////////////////////////////////////////////////////////////////////////////////
		
		void Loaded()
        {
			LoadVariables();
            LoadMessages();
			
			farmStarted = false;

			RegisterPerm("admin");
			
			if (configData.autoStart)
			{
				timer.Repeat(configData.intervalAutoStart * 60, 0, cmdFarming);
			}
        }
		
		////////////////////////////////////////////////////////////////////////////////////////
        ///     Конфиг
        ////////////////////////////////////////////////////////////////////////////////////////
		
		protected override void LoadDefaultConfig()
        {
            PrintWarning("Generating new configuration file for " + this.Title);
            var config = new ConfigData
            {
				autoStart = true,
				intervalAutoStart = 60f,
				chatProgressInterval = 2f,
				minPlayers = 0,
				giveGift = true,
				giveRandomAllGift = true,
				showWinnerInChat = true,
				writeWinLog = false,
				showWinConsole = false,
				listGiftItems = new List<GiftItems> 
				{
					{
						new GiftItems
						{
							itemName = "Detonator Cap",
							itemID = 231,
							itemCount = 1
						}
					},
					{
						new GiftItems
						{
							itemName = "Silnik 1100 (car)",
							itemID = 173,
							itemCount = 1
						}
					},
					{
						new GiftItems
						{
							itemName = "Bursztyn",
							itemID = 87,
							itemCount = 100
						}
					},
					{
						new GiftItems
						{
							itemName = "Mina",
							itemID = 273,
							itemCount = 10
						}
					},
					{
						new GiftItems
						{
							itemName = "Tłuszcz zwierzęcy",
							itemID = 23,
							itemCount = 150
						}
					},
					{
						new GiftItems
						{
							itemName = "Ukształtowane Niebieskie",
							itemID = 136,
							itemCount = 100
						}
					},
					{
						new GiftItems
						{
							itemName = "Ukształtowane Zielone",
							itemID = 135,
							itemCount = 100
						}
					},
					{
						new GiftItems
						{
							itemName = "Ukształtowane Czerwone",
							itemID = 134,
							itemCount = 150
						}
					},
					{
						new GiftItems
						{
							itemName = "Dynamit",
							itemID = 155,
							itemCount = 25
						}
					}
				},
				listFarmEvents = new List<FarmEvents>
				{
					{
						new FarmEvents
						{
							Name = "wood255",
							Item = "Items/Wood Log",
							Time = 5f,
							Farm = 255,
							Places = 3
						}
					},
					{
						new FarmEvents
						{
							Name = "wood510",
							Item = "Items/Wood Log",
							Time = 10f,
							Farm = 510,
							Places = 3
						}
					},
					{
						new FarmEvents
						{
							Name = "stone255",
							Item = "Items/Stone",
							Time = 5f,
							Farm = 255,
							Places = 3
						}
					},
					{
						new FarmEvents
						{
							Name = "stone510",
							Item = "Items/Stone",
							Time = 10f,
							Farm = 510,
							Places = 3
						}
					}
				}
			};
            SaveConfig(config);
        }
		
		////////////////////////////////////////////////////////////////////////////////////////
        ///     Языковые Сообщения
        ////////////////////////////////////////////////////////////////////////////////////////
		
		void LoadMessages()
        {
			//Polski
			lang.RegisterMessages(new Dictionary<string, string>
            {
                {"msg_Prefix", "<color=#33ff99>[Farming]</color>"},
				{"msg_NoPermission", "<color=red>Nie posiadasz permisji do użycia tej komendy.</color>"},
				{"msg_Progress", "Aktualny postęp <color=#0099ff>dropków</color>:"},
				{"msg_Finnish", "Event <color=red>zakończony</color>, wyniki:"},
                {"msg_Winners", "<color=#0099ff>{num}</color>. <color=#0099ff>{name}</color> ukończył w <color=#0099ff>{time}</color>!"},
				{"msg_Winner", "<color=#0099ff>{name}</color> w nagrode otrzymuje <color=#0099ff>{gift}</color>(<color=#0099ff>x{cout}</color>)!"},
				{"msg_Farming", "<color=#0099ff>{num}</color>. <color=#0099ff>{name}</color> uzbierał <color=#0099ff>{amount}</color>!"},
                {"msg_Finished", "<color=#0099ff>{name}</color> uzbierał wszystko w <color=yellow>{time}</color>!"},
                {"msg_Start", "Event rozpoczęty musisz uzbierać <color=#0099ff>{amount}</color>w <color=#0099ff>{item}</color>!"},
				{"msg_Stop", "<color=red>Event zakończony!</color>"},
                {"msg_Started", "<color=#3399ff>Już event wystartował.</color>"},
                {"msg_Stopped", "<color=#3399ff>Event nie wystarował</color>"},
				{"msg_NoEvent", "<color=#3399ff>Nie istnieje event z taką nazwą!</color>"},
				{"msg_Commands", "Dostępne Komendy:"},
				{"msg_CMDStart", "<color=#ccff00>/farm start</color> - Start event."},
				{"msg_CMDStartName", "<color=#ccff00>/farm start <name></color> - start eventu z nazwą"},
				{"msg_CMDStop", "<color=#ccff00>/farm stop</color> - event zatrzymany"},
				{"msg_Minutes", " min."},
				{"msg_Seconds", " sek."},
				{"Stone", "Kamień"},
				{"Wood Log", "Kłody"},
				{"Flint", "Krzemień"},
				{"Limestone", "Wapień"},
				{"Clay", "Glina"},
				{"Coal", "Węgiel"},
				{"Amber", "Bursztyn"},
				{"Iron Ore", "Żelazo"},
				{"Ultranium Ore", "Ultranium"},
				{"Mondinium Ore", "Mondinium"},
				{"Titranium Ore", "Titranium"},
				{"Tree Bark", "Kora"}
            }, this);
        }
		
		//////////////////////////////////////////////////////////////////////////////////////////
        ///// Комманды
        //////////////////////////////////////////////////////////////////////////////////////////
		
		[ChatCommand("farm")]
		void cmdFarm(PlayerSession player, string command, string[] args)
		{
		    if (args.Length == 0)
		    {
                if (HasPerm(player.SteamId, "admin") || player.IsAdmin)
				{
					SendChatMessage(player, GetMsg("msg_Prefix"), GetMsg("msg_Commands", player.SteamId.ToString()));
					SendChatMessage(player, null, GetMsg("msg_CMDStart", player.SteamId.ToString()));
					SendChatMessage(player, null, GetMsg("msg_CMDStartName", player.SteamId.ToString()));
					SendChatMessage(player, null, GetMsg("msg_CMDStop", player.SteamId.ToString()));
					return;
				}
				else
				{
					SendChatMessage(player, GetMsg("msg_Prefix"), GetMsg("msg_NoPermission", player.SteamId.ToString()));
					return;
				}	
  		    }
			var Action = args[0];
   		    switch (Action)
   		    {
				case "start":
		            if (HasPerm(player.SteamId, "admin") || player.IsAdmin)
                    {
						if (args.Length == 2)
						{
							if(!farmStarted)
							{
								if(getFarming(args[1].ToString()) == null)
								{
									SendChatMessage(player, GetMsg("msg_Prefix"), GetMsg("msg_NoEvent", player.SteamId.ToString()));
									return;
								}
								cmdFarmingStart(args[1].ToString());
							}
							else
							{
								SendChatMessage(player, GetMsg("msg_Prefix"), GetMsg("msg_Started", player.SteamId.ToString()));
								return;
							}
						}
						else
						{
							if(!farmStarted)
							{
								cmdFarmingStart();
							}
							else
							{
								SendChatMessage(player, GetMsg("msg_Prefix"), GetMsg("msg_Started", player.SteamId.ToString()));
								return;
							}
						}
					}
					else
					{
						SendChatMessage(player, GetMsg("msg_Prefix"), GetMsg("msg_NoPermission", player.SteamId.ToString()));
						return;
					}
		        break;
		        case "stop":
					if (HasPerm(player.SteamId, "admin") || player.IsAdmin)
                    {
						if(farmStarted)
						{
							cmdFarmingStop();
						}
						else
						{
							SendChatMessage(player, GetMsg("msg_Prefix"), GetMsg("msg_Stopped", player.SteamId.ToString()));
							return;
						}
					}
					else
					{
						SendChatMessage(player, GetMsg("msg_Prefix"), GetMsg("msg_NoPermission", player.SteamId.ToString()));
						return;
					}
		        break;
				default:
					if (HasPerm(player.SteamId, "admin") || player.IsAdmin)
                    {
						SendChatMessage(player, GetMsg("msg_Prefix"), GetMsg("msg_Commands", player.SteamId.ToString()));
						SendChatMessage(player, null, GetMsg("msg_CMDStart", player.SteamId.ToString()));
						SendChatMessage(player, null, GetMsg("msg_CMDStartName", player.SteamId.ToString()));
						SendChatMessage(player, null, GetMsg("msg_CMDStop", player.SteamId.ToString()));
						return;
					}
					else
					{
						SendChatMessage(player, GetMsg("msg_Prefix"), GetMsg("msg_NoPermission", player.SteamId.ToString()));
						return;
					}
                break;
		    }
		}
		
		////////////////////////////////////////////////////////////////////////////////////////
        //    Запуск для авто-ивента
        ////////////////////////////////////////////////////////////////////////////////////////
		
		void cmdFarming()
        {
			if(!getMinPlayers())
			{
				return;
			}
			
			cmdFarmingStart();
            return;	  
        } 
		
		////////////////////////////////////////////////////////////////////////////////////////
        //    Запускаем ивент
        ////////////////////////////////////////////////////////////////////////////////////////
		
		void cmdFarmingStart(string name = null)
        {
			FarmEvents fevent;
			
			if(farmStarted)
			{
				return;
			}
			
			if(name != null)
			{
				fevent = getFarming(name);
			}
			else
			{
				int	rand = UnityEngine.Random.Range(0, configData.listFarmEvents.Count);
				fevent = configData.listFarmEvents[rand];
			}
			
			currentFarmEvent.Name = fevent.Name;
			currentFarmEvent.Item = fevent.Item;
			currentFarmEvent.Farm = fevent.Farm;
			currentFarmEvent.Places = fevent.Places;
			currentFarmEvent.Time = timer.Once(fevent.Time * 60, () => { cmdFarmingStop(); });
			currentFarmEvent.Progress = timer.Every(configData.chatProgressInterval * 60, () => { cmdFarmingStat(); });
			BroadcastChat(GetMsg("msg_Prefix"), GetMsg("msg_Start").Replace("{item}", GetMsg(currentFarmEvent.Item.ToString().Split('/').Last())).Replace("{amount}", currentFarmEvent.Farm.ToString()));
			farmStarted = true;
			startTime = DateTime.Now;
        }
		
		//////////////////////////////////////////////////////////////////////////////////////////
        ///// Останавливаем Ивент
        //////////////////////////////////////////////////////////////////////////////////////////
		
		void cmdFarmingStop(bool finnish = false)
        {
            if(finnish || playersWin.Count > 0)
			{
				cmdFarmingStat(true);
				if(configData.giveGift)
				{					
					GiftItems ItemGift = null;
					if(!configData.giveRandomAllGift)
					{
						int	rand = UnityEngine.Random.Range(0, configData.listGiftItems.Count);
						ItemGift = configData.listGiftItems[rand];
					}
					
					foreach (var win in playersWin)
					{
						if (IsValidSession(FindSession(win.Key.ToString())))
						{
							if(configData.giveRandomAllGift)
							{
								int	rand = UnityEngine.Random.Range(0, configData.listGiftItems.Count);
								ItemGift = configData.listGiftItems[rand];
							}
							string pName = "";
							pName = FindSession(win.Key.ToString()).Name;
							
							Singleton<GlobalItemManager>.Instance.GiveItem(FindSession(win.Key.ToString()).Player, Singleton<GlobalItemManager>.Instance.GetItem(ItemGift.itemID), ItemGift.itemCount);
							if (configData.showWinnerInChat)
							{
								BroadcastChat(GetMsg("msg_Prefix"), GetMsg("msg_Winner").Replace("{name}", pName).Replace("{gift}", ItemGift.itemName.ToString()).Replace("{cout}", ItemGift.itemCount.ToString()));
							}
							if (configData.writeWinLog)
							{
								LogToFile("WinLog", $"["+System.DateTime.Now+"] "+pName+" ("+FindSession(win.Key.ToString()).SteamId+") GIFT: "+ItemGift.itemName.ToString()+" ID: "+ItemGift.itemID.ToString()+" COUNT:"+ItemGift.itemCount.ToString(), this, true);
							}
							if (configData.showWinConsole)
							{
								Puts(pName+" ("+FindSession(win.Key.ToString()).SteamId+") GIFT: "+ItemGift.itemName.ToString()+" ID: "+ItemGift.itemID.ToString()+" COUNT:"+ItemGift.itemCount.ToString());
							}
						}
					}
				
				}
			}
			else
			{
				BroadcastChat(GetMsg("msg_Prefix"), GetMsg("msg_Stop"));
			}
            currentFarmEvent.Time.Destroy();
            currentFarmEvent.Progress.Destroy();
            currentFarmEvent = new FarmEvent();
            playersFarming.Clear();
            playersWin.Clear();
			farmStarted = false;
        }
		
		////////////////////////////////////////////////////////////////////////////////////////
        //    Добыча ресурсов игроком
        ////////////////////////////////////////////////////////////////////////////////////////
		
		void OnDispenserGather(GameObject resourceNode, HurtMonoBehavior player, List<ItemInstance> items)
		{
			string Name = GetNameOfObject(resourceNode);
			Name = Name.Replace("(P)","");
			PlayerSession session = FindSession(Name);
			if (session == null) return;
			if (farmStarted && !playersWin.ContainsKey(Name))
            {
				foreach (var item in items)
				{
					if (item?.Item == null) return;
					if (item.Item.GetNameKey().ToString() == currentFarmEvent.Item.ToString())
					{
						if (!playersFarming.ContainsKey((ulong)session.SteamId))
						{
							playersFarming.Add((ulong)session.SteamId, item.StackSize);
						}
						else
						{
							playersFarming[(ulong)session.SteamId] += item.StackSize;
						}
						
						if (playersFarming[(ulong)session.SteamId] >= currentFarmEvent.Farm)
						{
							if (playersWin.Count < currentFarmEvent.Places)
							{
								playersWin.Add(Name, DateTime.Now);
								TimeSpan ts = DateTime.Now - startTime;
								BroadcastChat(GetMsg("msg_Prefix"), GetMsg("msg_Finished").Replace("{name}", Name).Replace("{time}", ConvertSecondsToDate(ts.TotalSeconds)));
								playersFarming.Remove((ulong)session.SteamId);
								return;
							}
							else
							{
								cmdFarmingStop(true);
								return;
							}
						}
					}
				}
            }
		}
		
		////////////////////////////////////////////////////////////////////////////////////////
        //    Отображения прогресса и результата ивента
        ////////////////////////////////////////////////////////////////////////////////////////
		
		void cmdFarmingStat(bool finnish = false)
        {
			if(finnish)
			{
				BroadcastChat(GetMsg("msg_Prefix"), GetMsg("msg_Finnish"));
			}
			else
			{
				if(playersFarming.Count > 0 || playersWin.Count > 0)
				{
					BroadcastChat(GetMsg("msg_Prefix"), GetMsg("msg_Progress"));
				}
			}
            int i = 1;
            foreach (var wins in playersWin)
            {
                TimeSpan ts = wins.Value - startTime;
				BroadcastChat(null, GetMsg("msg_Winners").Replace("{num}", i.ToString()).Replace("{name}", wins.Key).Replace("{time}", ConvertSecondsToDate(ts.TotalSeconds)));
                i++;
            }
			foreach (var farm in playersFarming.OrderByDescending(p => p.Value).Take(currentFarmEvent.Places - i + 1))
            {
				if (IsValidSession(FindSession(farm.Key.ToString())))
				{
					string pName = "";
					pName = FindSession(farm.Key.ToString()).Name;
					
					BroadcastChat(null, GetMsg("msg_Farming").Replace("{num}", i.ToString()).Replace("{name}", pName).Replace("{amount}", farm.Value.ToString()));
					i++;
				}
            }
        }
		
		////////////////////////////////////////
        ///     Время
        ////////////////////////////////////////
		
		private string ConvertSecondsToDate(double numOfSeconds)
        {
            int minutes = (int) (numOfSeconds / 60) % 60;
            int seconds = (int)numOfSeconds  % 60;
			
			string SecondsToDate = "";
			string sMinutes = "";
			string sSeconds = "";
			
			if (minutes > 0) {
                sMinutes = minutes.ToString() + GetMsg("msg_Minutes") + " ";
		    }
			
			if (seconds > 0) {
                sSeconds = seconds.ToString() + GetMsg("msg_Seconds");
			}

            SecondsToDate = sMinutes + sSeconds;
			if (SecondsToDate.EndsWith(" "))
				SecondsToDate = SecondsToDate.Substring(0, SecondsToDate.Length - 1);
			
			return SecondsToDate;
        }
		
		////////////////////////////////////////////////////////////////////////////////////////
        //    Достаем все данные нужного нам ивента фарма
        ////////////////////////////////////////////////////////////////////////////////////////
		
		FarmEvents getFarming(string name)
		{
			FarmEvents farme = (from x in configData.listFarmEvents where x.Name == name select x).FirstOrDefault();
			return farme;		
		}
		
		////////////////////////////////////////////////////////////////////////////////////////
        ///     АПИ / API
        ////////////////////////////////////////////////////////////////////////////////////////
		
		bool isEventFarming()
		{
			return farmStarted;
		}
		
		////////////////////////////////////////////////////////////////////////////////////////
        //    Проверяем достаточное ли количество игроков онлайн 
        ////////////////////////////////////////////////////////////////////////////////////////
		
		bool getMinPlayers()
        {
            var allPlayers = GameManager.Instance.GetSessions().Values.ToList();
            if (allPlayers.Count() >= configData.minPlayers)
            {
                return true;
            }
            else
            {
                return false;
            }
        }
		
		////////////////////////////////////////////////////////////////////////////////////////
        //    Узнаем название(имя) игрового обекта
        ////////////////////////////////////////////////////////////////////////////////////////
		
		string GetNameOfObject(UnityEngine.GameObject obj)
        {
            var ManagerInstance = GameManager.Instance;
            return ManagerInstance.GetDescriptionKey(obj);
        }
		
		////////////////////////////////////////////////////////////////////////////////////////
        //    Находим игровую сессию по нику или стим ид
        ////////////////////////////////////////////////////////////////////////////////////////
		
		private PlayerSession FindSession(string nameOrIdOrIp)
        {
            var sessions = GameManager.Instance.GetSessions();
            PlayerSession session = null;
            foreach (var i in sessions)
            {
				if (nameOrIdOrIp.Equals(i.Value.Name, StringComparison.OrdinalIgnoreCase) ||
					nameOrIdOrIp.Equals(i.Value.SteamId.ToString()) || nameOrIdOrIp.Equals(i.Key.ipAddress))
				{
					session = i.Value;
					break;
				}
            }
            return session;
        }
		
		////////////////////////////////////////
        ///     Проверка Сессии
        ////////////////////////////////////////
		
		public bool IsValidSession(PlayerSession session)
		{
			return session?.SteamId != null && session.IsLoaded && session.Name != null && session.Identity != null &&
				   session.WorldPlayerEntity?.transform?.position != null;
		}
		
		////////////////////////////////////////
        ///     Converting
        ////////////////////////////////////////

        string ListToString(List<string> list, int first, string seperator) => string.Join(seperator, list.Skip(first).ToArray());
		
		//////////////////////////////////////////////////////////////////////////////////////////
        /////      Функции Данных конфига, "запись(замена)" и "чтение"...
        //////////////////////////////////////////////////////////////////////////////////////////
		
        private void LoadVariables()
        {
            LoadConfigVariables();
            SaveConfig();
        }
		
        private void LoadConfigVariables()
        {
            configData = Config.ReadObject<ConfigData>();
        }
		
		private void SaveConfig(ConfigData config)
        {
            Config.WriteObject(config, true);
        }

        /////////////////////////////////////////////////////////////////////////////////////////
		
		////////////////////////////////////////////////////////////////////////////////////////
        ///     Разрешения (permissions, пермишены)
        ////////////////////////////////////////////////////////////////////////////////////////

        void RegisterPerm(params string[] permArray)
        {
            string perm = ListToString(permArray.ToList(), 0, ".");

            permission.RegisterPermission($"{PermissionPrefix}.{perm}", this);
        }

        bool HasPerm(object uid, params string[] permArray)
        {
            uid = uid.ToString();
            string perm = ListToString(permArray.ToList(), 0, ".");

            return permission.UserHasPermission(uid.ToString(), $"{PermissionPrefix}.{perm}");
        }

        string PermissionPrefix
        {
            get
            {
                return this.Title.Replace(" ", "").ToLower();
            }
        }
		
		//////////////////////////////////////////////////////////////////////////////////////
        ///     ЧАТ
        //////////////////////////////////////////////////////////////////////////////////////
		
		string GetMsg(string key, string userID = "") => lang.GetMessage(key, this, userID);
		
		void BroadcastChat(string prefix, string msg) => Server.Broadcast(msg, prefix);

        void SendChatMessage(PlayerSession player, string prefix, string msg) => hurt.SendChatMessage(player, prefix, msg);
		
	}
}